import * as THREE from "three"
import { useEffect, useState } from 'react'
import { GLTFLoader } from "three/examples/jsm/loaders/GLTFLoader";
import { DRACOLoader } from "three/examples/jsm/loaders/DRACOLoader";
import ModelDuplicate from "./modelDuplicate";
import { useLoadingProgress } from "../../../../store/builder/store";
import { getFloorAssets } from "../../../../services/factoryBuilder/assest/floorAsset/getFloorItemsApi";
import { FloorItems } from "../../../../types/world/worldTypes";

const gltfLoaderWorker = new Worker(
    new URL(
        "../../../../services/factoryBuilder/webWorkers/gltfLoaderWorker.js",
        import.meta.url
    )
);

function AssetsGroupDuplicate({ projectId }: { projectId: string }) {
    const { setLoadingProgress } = useLoadingProgress();
    const email = localStorage.getItem("email");
    const organization = email!.split("@")[1].split(".")[0];
    const [assetsDuplicates, setAssetsDuplicates] = useState<Assets>([]);

    const loader = new GLTFLoader();
    const dracoLoader = new DRACOLoader();

    dracoLoader.setDecoderPath(
        "https://cdn.jsdelivr.net/npm/three@0.160.0/examples/jsm/libs/draco/gltf/"
    );
    loader.setDRACOLoader(dracoLoader);

    useEffect(() => {
        let totalAssets = 0;
        let loadedAssets = 0;

        const updateLoadingProgress = (progress: number) => {
            if (progress < 100) {
                setLoadingProgress(progress);
            } else if (progress === 100) {
                setTimeout(() => {
                    setLoadingProgress(100);
                    setTimeout(() => {
                        setLoadingProgress(0);
                    }, 1500);
                }, 1000);
            }
        };

        getFloorAssets(organization, projectId).then((data) => {
            if (data.length > 0) {
                const uniqueItems = (data as FloorItems).filter((item, index, self) => index === self.findIndex((t) => t.assetId === item.assetId));
                totalAssets = uniqueItems.length;
                if (totalAssets === 0) {
                    updateLoadingProgress(100);
                    return;
                }
                gltfLoaderWorker.postMessage({ floorItems: uniqueItems });
            } else {
                gltfLoaderWorker.postMessage({ floorItems: [] });
                updateLoadingProgress(100);
            }
        });

        gltfLoaderWorker.onmessage = async (event) => {
            if (event.data.message === "gltfLoaded" && event.data.modelBlob) {
                const blobUrl = URL.createObjectURL(event.data.modelBlob);
                loader.load(blobUrl, (gltf) => {
                    URL.revokeObjectURL(blobUrl);
                    THREE.Cache.remove(blobUrl);
                    THREE.Cache.add(event.data.modelID, gltf);

                    loadedAssets++;
                    const progress = Math.round((loadedAssets / totalAssets) * 100);
                    updateLoadingProgress(progress);

                    if (loadedAssets === totalAssets) {
                        getFloorAssets(organization, projectId).then((data: FloorItems) => {
                            const newAssets: Assets = data.map((item) => {
                                if (item.eventData) {
                                    return {
                                        modelUuid: item.modelUuid,
                                        modelName: item.modelName,
                                        assetId: item.assetId,
                                        position: item.position,
                                        rotation: [item.rotation.x, item.rotation.y, item.rotation.z],
                                        isLocked: item.isLocked,
                                        isCollidable: false,
                                        isVisible: item.isVisible,
                                        opacity: 1,
                                        eventData: item.eventData
                                    };
                                } else {
                                    return {
                                        modelUuid: item.modelUuid,
                                        modelName: item.modelName,
                                        assetId: item.assetId,
                                        position: item.position,
                                        rotation: [item.rotation.x, item.rotation.y, item.rotation.z],
                                        isLocked: item.isLocked,
                                        isCollidable: false,
                                        isVisible: item.isVisible,
                                        opacity: 1,
                                    };
                                }
                            });
                            setAssetsDuplicates(newAssets);
                        });
                        updateLoadingProgress(100);
                    }
                });
            }
        };
    }, []);

    return (
        <>
            {assetsDuplicates.map((asset) =>
                <ModelDuplicate key={`${asset.modelUuid}_Duplicate`} asset={asset} />
            )}
        </>
    )
}

export default AssetsGroupDuplicate;